// @ myth - common - command.C
//
// Myth is a multiuser screen oriented adventure game.
// Copyright (C) 1989 Per Abrahamsen
// 
// This file is part of Myth.
// 
// Myth is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 1, or (at your option) any later
// version.
// 
// Myth is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Myth; see the file COPYING.  If not, write to the Free
// Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
// My email address is: amanda@iesd.auc.dk

// @@ include

#include "common/command.h"
#include "common/common.h"
#include "common/error.h"

// @@ CommandToType

// @@@ static const char* CommandName[]

static const char* CommandName[] =
{ "IDLE", "MOVE", "RUN", "DROP", "GET", "APPLY", "THROW", "SLEEP",
  "COMMAND", "RECENTER", "INVENTORY", "STATUS", "NEWS", "UP", "DOWN", NULL };

// @@@ from_client CommandToType(const char* name)
//
// This look name up in the table CommandName, and return the
// corresponding type.
//
// It should be called like CommandName.type(name), but c++ don't accept
// initialised members of classes yet.

from_client CommandToType(const char* name)
{
    for(from_client command = 0; command < CLIENT_END; command++)
	if(strcmp(name, CommandName[command]) == EQUAL)
	    return command;

    return CLIENT_ERROR;
}

// @@ StatusToType

// @@@ static const char* StatusName[]

static const char* StatusName[] =
{ "NEWS",
  "EXPERIENCE", "DUNGEON", "GAME",
  "STRENGTH", "HEALTH",
  "FOOD",
  NULL
};

// @@@ status_type StatusToType(const char* name)
//
// This look name up in the table StatusName, and return the
// corresponding type.
//
// It should be called like StatusName.type(name), but c++ don't accept
// initialised members of classes yet.

status_type StatusToType(const char* name)
{
    for(status_type status = 0; status < STATUS_END; status++)
	if(strcmp(name, StatusName[status]) == EQUAL)
	    return status;

    return STATUS_ERROR;
}

// @@ RequestToType

// @@@ static const char* RequestName[]

static const char* RequestName[] =
{ "IDLE", "OBJECT", "POSITION", "STRING", "DIRECTION", "SYMBOL",
  "CONFIRMATION", "ARGUMENT", "LETTER", "NAME", "COMMAND", NULL };

// @@@ request_type RequestToType(const char* name)
//
// This look name up in the table RequestName, and return the
// corresponding type.
//
// It should be called like RequestName.type(name), but c++ don't accept
// initialised members of classes yet.

request_type RequestToType(const char* name)
{
    for(request_type request = 0; request < REQUEST_END; request++)
	if(strcmp(name, RequestName[request]) == EQUAL)
	    return request;

    return REQUEST_ERROR;
}

// @@ PromptToType

// @@@ static const char* PromptName[]

static const char* PromptName[] =
{ "IDLE",
  "SYMBOL", "LETTER", "POSITION", "STRING",
  "DIRECTION", "CONFIRMATION", "ARGUMENT", "NAME",
  "COMMAND",
  "COPYRIGHT", "WELCOME", "PASSWORD",
  "IDENTIFY", "TELEPORT", "BLESS", "MAGIC",
  "ZAP", "BREATHE", "EAT",
  "MOVE", "RUN", "DROP", "GET", "APPLY",
  "THROW", "SLEEP", "EXTENDED", "RECENTER",
  "INVENTORY",
  NULL };

// @@@ prompt_type PromptToType(const char* name)
//
// This look name up in the table PromptName, and return the
// corresponding type.
//
// It should be called like PromptName.type(name), but c++ don't accept
// initialised members of classes yet.

prompt_type PromptToType(const char* name)
{
    for(prompt_type prompt = 0; prompt < PROMPT_END; prompt++)
	if(strcmp(name, PromptName[prompt]) == EQUAL)
	    return prompt;

    return PROMPT_ERROR;
}

// @@ emacs
//
// Local Variables:
// mode: c++
// outline-regexp: "// @+ "
// eval: (outline-minor-mode 1)
// End:
