// @ myth - common - string.h
//
// Myth is a multiuser screen oriented adventure game.
// Copyright (C) 1989 Per Abrahamsen
// 
// This file is part of Myth.
// 
// Myth is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 1, or (at your option) any later
// version.
// 
// Myth is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Myth; see the file COPYING.  If not, write to the Free
// Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
// My email address is: amanda@iesd.auc.dk

// @@ begin

#ifndef STRING_HEADER
#define STRING_HEADER

// @@ include

#include "common/common.h"

// @@ enum string_type

enum string_type
{ STRING_COPY, STRING_STATIC };

// @@ (local) string_link

struct string_link
{
    const char* string;		// Pointer to the real string.
    int length;			// The length of the string.
    int count;			// Number of references.
    string_type type;		// Does the string point to a static area?
    string_link(const char* s, int l, string_type t);
    ~string_link();
};

// @@ STRING

class STRING
{
    string_link* link;
    void UnLink();
    void Link(const STRING* copy);
public:
    inline const char* string()
    { return (link == NULL) ? "" : link->string; }
    inline const char* operator()()
    { return string(); }
    inline int length()
    { return (link == NULL) ? 0 : link->length; }
    const STRING& operator=(const STRING& copy);
    bool operator==(const STRING& other);
    bool operator!=(const STRING& other);
    STRING(const char* string, string_type type = STRING_COPY);
    STRING(const STRING& copy);
    STRING();
    ~STRING();
};

// @@ STRONG
//
// A static version of STRING

class STRONG : public STRING
{
public:
    STRONG(const char* string);
};

// @@ const STRING NullString

static const STRING NullString = STRING();

// @@ end

#endif STRING_HEADER

// @@ emacs
//
// Local Variables:
// mode: c++
// outline-regexp: "// @+ "
// eval: (outline-minor-mode 1)
// End:
