// @ myth - server - ability.C
//
// Myth is a multiuser screen oriented adventure game.
// Copyright (C) 1989 Per Abrahamsen
// 
// This file is part of Myth.
// 
// Myth is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 1, or (at your option) any later
// version.
// 
// Myth is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Myth; see the file COPYING.  If not, write to the Free
// Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
// My email address is: amanda@iesd.auc.dk

// @@ include

#include "common/common.h"
#include "common/error.h"
#include "common/binary.h"
#include "server/level.h"
#include "server/monster.h"
#include "server/ability.h"
#include "server/rnd.h"
#include "server/beast.h"

// @@ COUNTER

// @@@ int COUNTER::value()

int COUNTER::value()
{
    int tmp = MyLog(count);

    if(permanent)
        return max(MyValue, tmp);
    else
        return tmp;
}

// @@@ void COUNTER::add(COUNTER* counter)

void COUNTER::add(COUNTER* counter)
{
    if(ability() == counter->ability())
    {
        if(counter->permanent)
        {
    	if(permanent)
    	    MyValue = max(MyValue, counter->MyValue);
    	else
    	    MyValue = counter->MyValue;
    	
    	permanent = TRUE;
        }
        count += counter->count;
    }
}

// @@@ cont COUNTER::tick(LEVEL* level, MONSTER* monster)

cont COUNTER::tick(LEVEL* level, MONSTER* monster)
{
    if(count > 0L)
        count--;

    if(permanent)
        return CONTINUE;
    else if(count < 1L)
    {
        effect(level, monster);
        return DONE;
    }
    else
        return CONTINUE;
}

// @@@ void COUNTER::remove(force flag)

void COUNTER::remove(force flag)
{
    if(count > 0 && flag == CHECK)
    {
	done = TRUE;
	count = 1;
    }
    else if(permanent && flag == FORCE)
    {
	if(count < 1)
	{
	    count = 1;
	    done = TRUE;
	}
	permanent = FALSE;
    }
}

// @@@ void COUNTER::save(BINARY* binary)

void COUNTER::save(BINARY* binary)
{
    binary->PutShort(MyAbility);
    binary->PutLong(count);
    binary->PutChar(permanent);
    binary->PutShort(MyValue);
}

// @@@ void COUNTER::load(BINARY* binary)

void COUNTER::load(BINARY* binary)
{
    count = binary->GetLong();
    permanent = binary->GetChar();
    MyValue = binary->GetShort();
}

// @@@ COUNTER::COUNTER(ability_type ability, long cnt, int val = 0)

COUNTER::COUNTER(ability_type ability, long cnt, int val = 0)
{
    count = cnt;
    permanent = (cnt == 0L);
    done = FALSE;
    MyValue = val;
    MyAbility = ability;
    touched = TRUE;
}

// @@ COUNTER_STONED

// @@@ cont COUNTER_STONED::tick(LEVEL* level, MONSTER* monster)

cont COUNTER_STONED::tick(LEVEL* level, MONSTER* monster)
{
    if(count > 0L)
        count--;

    switch(max(0, count--))
    {
    case 5:
	monster->message("You are slowing down.");
	break;
    case 4:
	monster->message("Your limbs are stiffening.");
	break;
    case 2:
	monster->message("Your limbs have turned to stone.");
	break;
    case 1:
	monster->message("You have turned to stone.");
	break;
    case 0:
	if(done)
	{
	    monster->message(
	        "What a pity - you just destroyed a future piece of art!"
			     );
	}
	else
	{
	    monster->message("You are a statue!");
	    level->WatchMonster(monster, "%s is now a statue!");
	    monster->kill("turned to stone", level);
	    // You should actually turn him to stone here
	    // Maybe a flag in body? IsStatue?
	}
	return DONE;
    }
    return CONTINUE;
}

// @@ COUNTER_INFECTED

// @@@ void COUNTER_INFECTED::effect(LEVEL* level, MONSTER* monster)

void COUNTER_INFECTED::effect(LEVEL* level, MONSTER* monster)
{
    if(done || rnd.chance(2, 3))
	monster->message("You feel the infection vanish.");
    else
    {
	FUNGUS* fungus = new FUNGUS;
	BUNCH* bunch = level->create(monster->position());
	bunch->add(fungus);
	level->AddMonster(fungus);
	monster->message("You die from an infection.");
	level->WatchMonster(monster, "%s dies from an infection.");
	monster->kill("was killed by a fungus infection", level);
    }
}

// @@ COUNTER_PHASING

// @@@ cont COUNTER_PHASING::tick(LEVEL* level, MONSTER* monster)

cont COUNTER_PHASING::tick(LEVEL* level, MONSTER* monster)
{
    if(count > 0L)
        count--;

    if(permanent)
	return CONTINUE;
    
    switch(max(0, count--))
    {
    case 5:
	monster->message("You are starting to become solid again.");
	break;
    case 0:
	monster->message("You are now solid again.");
	monster->move(level, monster->position());
	return DONE;
    }
    return CONTINUE;
}

// @@ COUNTER_SICK

// @@@ void COUNTER_SICK::effect(LEVEL* level, MONSTER* monster)

void COUNTER_SICK::effect(LEVEL* level, MONSTER* monster)
{
    if(done)
	monster->message("You feel your health returning.");
    else
    {
	monster->message("You die because of food poisoning.");
	level->WatchMonster(monster, "%s dies because of food poisoning.");
	monster->kill("was killed by an old cadaver", level);
    }
}

// @@ COUNTER_CONFUSED

// @@@ int COUNTER_CONFUSED::value()

int COUNTER_CONFUSED::value()
{
    int tmp = count / 20 + 1;

    if(permanent)
        return max(MyValue, tmp);
    else
        return tmp;
}

// @@@ void COUNTER_CONFUSED::effect(LEVEL*, MONSTER* monster)

void COUNTER_CONFUSED::effect(LEVEL*, MONSTER* monster)
{
    monster->message("You feel less confused now.");
}

// @@ COUNTER_BLIND

// @@@ void COUNTER_BLIND::effect(LEVEL*, MONSTER* monster)

void COUNTER_BLIND::effect(LEVEL*, MONSTER* monster)
{
    monster->message("You can see again!");
}

// @@ COUNTER_SIGHT

// @@@ int COUNTER_SIGHT::value()

int COUNTER_SIGHT::value()
{
    int tmp = count / 100 + 1;

    if(permanent)
        return max(MyValue, tmp);
    else
        return tmp;
}

// @@ COUNTER_INVISIBLE

// @@@ void COUNTER_INVISIBLE::effect(LEVEL*, MONSTER* monster)

void COUNTER_INVISIBLE::effect(LEVEL*, MONSTER* monster)
{
    if(monster->SeeInvisible())
	monster->message("You feel insecure...");
    else
	monster->message("You are no longer invisible.");
}

// @@ COUNTER_FAST

// @@@ int COUNTER_FAST::value()

int COUNTER_FAST::value()
{
    int tmp = count / 2;

    if(permanent)
        return max(MyValue, tmp);
    else
        return tmp;
}

// @@@ void COUNTER_FAST::effect(LEVEL*, MONSTER* monster)

void COUNTER_FAST::effect(LEVEL*, MONSTER* monster)
{
    monster->message("You are slowing down.");
}

// @@ COUNTER_SLOW

// @@@ void COUNTER_SLOW::effect(LEVEL*, MONSTER* monster)

void COUNTER_SLOW::effect(LEVEL*, MONSTER* monster)
{
    monster->message("You are speeding up.");
}

// @@ COUNTER_LIGHT

// @@@ void COUNTER_LIGHT::effect(LEVEL*, MONSTER* monster)

void COUNTER_LIGHT::effect(LEVEL*, MONSTER* monster)
{
    monster->message("You are really turned off by now.");
}

// @@ COUNTER* ::LoadCounter(BINARY* binary)

COUNTER* LoadCounter(BINARY* binary)
{
    int ability = binary->GetShort();
    COUNTER* counter;
    
    switch(ability)
    {
    case ABILITY_STONED:
	counter = new COUNTER_STONED(5L);
	break;
    case ABILITY_INFECTED:
	counter = new COUNTER_INFECTED(5L);
	break;
    case ABILITY_SICK:
	counter = new COUNTER_SICK(5L);
	break;
    case ABILITY_CONFUSED:
	counter = new COUNTER_CONFUSED;
	break;
    case ABILITY_BLIND:
	counter = new COUNTER_BLIND;
	break;
    case ABILITY_SIGHT:
	counter = new COUNTER_SIGHT;
	break;
    case ABILITY_SEE_INVISIBLE:
	counter = new COUNTER_SEE_INVISIBLE;
	break;
    case ABILITY_INVISIBLE:
	counter = new COUNTER_INVISIBLE;
	break;
    case ABILITY_DETECT:
	counter = new COUNTER_DETECT;
	break;
    case ABILITY_FAST:
	counter = new COUNTER_FAST;
	break;
    case ABILITY_SLOW:
	counter = new COUNTER_SLOW;
	break;
    case ABILITY_LIGHT:
	counter = new COUNTER_LIGHT;
	break;
    case ABILITY_TELEPORT_CONTROL:
	counter = new COUNTER_TELEPORT_CONTROL;
	break;
    case ABILITY_RANDOM_TELEPORT:
	counter = new COUNTER_RANDOM_TELEPORT;
	break;
    case ABILITY_HEALTH:
	counter = new COUNTER_HEALTH;
	break;
    case ABILITY_HUNGER:
	counter = new COUNTER_HUNGER;
	break;
    case ABILITY_STRENGTH:
	counter = new COUNTER_STRENGTH;
	break;
    case ABILITY_YENDOR:
	counter = new COUNTER_YENDOR;
	break;
    case ABILITY_FIRE_PROTECTION:
	counter = new COUNTER_FIRE_PROTECTION;
	break;
    case ABILITY_COLD_PROTECTION:
	counter = new COUNTER_COLD_PROTECTION;
	break;
    case ABILITY_ACID_PROTECTION:
	counter = new COUNTER_ACID_PROTECTION;
	break;
    case ABILITY_STONE_PROTECTION:
	counter = new COUNTER_STONE_PROTECTION;
	break;
    case ABILITY_POISON_PROTECTION:
	counter = new COUNTER_POISON_PROTECTION;
	break;
    case ABILITY_DEATH_PROTECTION:
	counter = new COUNTER_DEATH_PROTECTION;
	break;
    case ABILITY_HISS:
	counter = new COUNTER_HISS;
	break;
    case ABILITY_BREATHE_FIRE:
	counter = new COUNTER_BREATHE_FIRE;
	break;
    case ABILITY_END:
	return NULL;
    case ABILITY_ERROR:
    default:
	bug.warning("::LoadCounter", "Bad data");
	return NULL;
    }
    counter->load(binary);
    return counter;
}

// @@ COUNTER_LIST

// @@@ void COUNTER_LIST::insert(COUNTER* counter)

void COUNTER_LIST::insert(COUNTER* counter)
{
    ability_type ability = counter->ability();
    COUNTER* entry = table[ability];
    
    if(entry == NULL)
    {
        table[ability] = counter;
        list.add((void*) counter);
    }
    else
    {
        entry->add(counter);
        delete counter;
    }
}

// @@@ void COUNTER_LIST::update(LEVEL* level, MONSTER* monster)

void COUNTER_LIST::update(LEVEL* level, MONSTER* monster)
{
    list.reset();
    while(list.current() != NULL)
    {
        COUNTER* counter = (COUNTER*) list.current();

        if(counter->tick(level, monster) == DONE)
        {
	    table[counter->ability()] = NULL;
	    delete counter;
	    list.remove();
        }
        else
	    list.next();
    }
}

// @@@ void COUNTER_LIST::remove(ability_type ability, force flag)

void COUNTER_LIST::remove(ability_type ability, force flag)
{
    if(table[ability] != NULL)
	table[ability]->remove(flag);
}

// @@@ void COUNTER_LIST::clear()

void COUNTER_LIST::clear()
{
    list.reset();
    while(list.current() != NULL)
    {
	COUNTER* counter = (COUNTER*) list.current();

	table[counter->ability()] = NULL;
	delete counter;
	list.remove();
    }
}

// @@@ void COUNTER_LIST::save(BINARY* binary)

void COUNTER_LIST::save(BINARY* binary)
{
    list.reset();
    while(list.current() != NULL)
    {
        ((COUNTER*) list.current())->save(binary);
        list.next();
    }
    binary->PutShort(ABILITY_END);
}

// @@@ void COUNTER_LIST::load(BINARY* binary)

void COUNTER_LIST::load(BINARY* binary)
{
    COUNTER* counter;

    while((counter = ::LoadCounter(binary)) != NULL)
        insert(counter);
}

// @@@ COUNTER_LIST::COUNTER_LIST()

COUNTER_LIST::COUNTER_LIST()
{
    for(ability_type ability = 0; ability < ABILITY_END; ability++)
        table[ability] = NULL;
}

// @@@ COUNTER_LIST::~COUNTER_LIST()

COUNTER_LIST::~COUNTER_LIST()
{
    list.reset();
    while(list.current() != NULL)
    {
        COUNTER* counter = (COUNTER*) list.current();
        table[counter->ability()] = NULL;
        delete counter;
        list.remove();
    }
}

// @@ diretion ::ConfuseDirection(direction dir)
//
direction ConfuseDirection(direction dir)
{
    switch(rnd(1, 3))
    {
    case 1:
	return DIRECTION_NOWHERE + rnd(1, 8);
    case 2:
	if(dir == DIRECTION_NOWHERE + 8)
	    return DIRECTION_NOWHERE + 1;
	else
	    return dir + 1;
    case 3:
	if(dir == DIRECTION_NOWHERE + 1)
	    return DIRECTION_NOWHERE + 8;
	else
	    return dir - 1;
    default:
	bug.error("::ConfuseDirection", "rnd");
    }
    return DIRECTION_ERROR;
}

// @@ emacs
//
// Local Variables:
// mode: c++
// outline-regexp: "// @+ "
// eval: (outline-minor-mode 1)
// End:
