// @ myth - server - command.h
//
// Myth is a multiuser screen oriented adventure game.
// Copyright (C) 1989 Per Abrahamsen
// 
// This file is part of Myth.
// 
// Myth is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 1, or (at your option) any later
// version.
// 
// Myth is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Myth; see the file COPYING.  If not, write to the Free
// Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
// My email address is: amanda@iesd.auc.dk

// @@ begin

// Not very bright... Two header files with the same name.
// #define COMMAND_HEADER

#ifndef HEADER_COMMAND
#define HEADER_COMMAND

// @@ include

#include "common/ipc.h"
#include "common/symbol.h"
#include "common/command.h"
#include "common/string.h"

struct LEVEL;
struct PLAYER;
struct QUESTION;

// @@ enum question_state

enum question_state
{
    Q_STATE_ESCAPE = -5,	// User escaped from the question
    Q_STATE_ANSWER = -4,	// Waiting for an answer
    Q_STATE_ARGUMENT = -3,	// Waiting for an argument
    Q_STATE_COMMAND = -2,	// Waiting for a command
    Q_STATE_DONE = -1,		// The command is parsed
    Q_STATE_START = 0		// Value given to the command for parsing
	// All other values may be used by the command for it's own parsing
};

// @@ class COMMAND

class COMMAND
{
protected:
    QUESTION* answer;
    void FollowUp(QUESTION* question)
    { answer = question; /* Should not be generalised... */ }
public:
    virtual question_state parse(CHANNEL*, question_state);
    virtual cont play(LEVEL*, PLAYER*, long&);
    QUESTION* next()
    { return answer; }
    COMMAND()
    { answer = NULL; }
    virtual ~COMMAND() { }
};

// @@ class COMMAND_DIRECTION
//
// Common parse() for commands requiring a direction.

class COMMAND_DIRECTION : public COMMAND
{
protected:
    direction dir;
public:
    question_state parse(CHANNEL* channel, question_state state);
};

// @@ class COMMAND_OBJECT
//
// Common parse() for commands requiring an object.

class COMMAND_OBJECT : public COMMAND
{
protected:
    symbol_type symbol;
    char letter;
public:
    question_state parse(CHANNEL* channel, question_state state);
};

// @@ class COMMAND_MOVE
    
class COMMAND_MOVE : public COMMAND_DIRECTION
{
public:
    cont play(LEVEL* level, PLAYER* player, long& argument);
};

// @@ class COMMAND_RUN

enum run_state
{ RUN_START, RUN_AUTO, RUN_ESCAPE, RUN_DIG, RUN_FIGHT };

class COMMAND_RUN : public COMMAND_DIRECTION
{
    run_state state;		// TRUE, if player is running
public:
    cont play(LEVEL* level, PLAYER* player, long& argument);
    COMMAND_RUN()
    { state = RUN_START; }
};

// @@ class COMMAND_DROP

class COMMAND_DROP : public COMMAND_OBJECT
{
public:
    cont play(LEVEL* level, PLAYER* player, long& argument);
};

// @@ class COMMAND_GET

class COMMAND_GET : public COMMAND
{
public:
    question_state parse(CHANNEL*, question_state);
    cont play(LEVEL* level, PLAYER* player, long& argument);
};

// @@ class COMMAND_APPLY

class COMMAND_APPLY : public COMMAND_OBJECT
{
public:
    cont play(LEVEL* level, PLAYER* player, long& argument);
};

// @@ class COMMAND_THROW

class COMMAND_THROW : public COMMAND
{
    symbol_type symbol;
    char letter;
    coor position;
public:
    question_state parse(CHANNEL* channel, question_state state);
    cont play(LEVEL* level, PLAYER* player, long& argument);
};

// @@ class COMMAND_SLEEP

class COMMAND_SLEEP : public COMMAND
{
    confirmation conf;
public:
    question_state parse(CHANNEL* channel, question_state state);
    cont play(LEVEL* level, PLAYER* player, long&);
};

// @@ class COMMAND_COMMAND

class COMMAND_COMMAND : public COMMAND
{
    char string[MessageSize];
public:
    question_state parse(CHANNEL* channel, question_state state);
    cont play(LEVEL* level, PLAYER* player, long&);
};

// @@ end

#endif HEADER_COMMAND

// @@ emacs
//
// Local Variables:
// mode: c++
// outline-regexp: "// @+ "
// eval: (outline-minor-mode 1)
// End:
